<?php

namespace Drupal\shopify_app;

use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Utility\Error;
use Shopify\Webhooks\Handler;

/**
 * The webhook handler.
 */
class WebhookHandler implements Handler {

  /**
   * Webhook plugin manager.
   *
   * @var \Drupal\shopify_app\WebhookPluginManager
   */
  protected $webhookPluginManager;

  /**
   * The logger channel factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerChannelFactory;

  /**
   * Constructor.
   *
   * @param \Drupal\shopify_app\WebhookPluginManager $webhookPluginManager
   *   The webhook plugin manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerChannelFactory
   *   The logger channel factory.
   */
  public function __construct(WebhookPluginManager $webhookPluginManager, LoggerChannelFactoryInterface $loggerChannelFactory) {
    $this->webhookPluginManager = $webhookPluginManager;
    $this->loggerChannelFactory = $loggerChannelFactory;
  }

  /**
   * Handle a webhook.
   *
   * @param string $topic
   *   The topic.
   * @param string $shop
   *   The shop.
   * @param array $body
   *   The request body.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function handle(string $topic, string $shop, array $body): void {
    foreach ($this->webhookPluginManager->getDefinitions() as $definition) {
      /**
       * @var \Drupal\shopify_app\WebhookPluginInterface $plugin
       */
      $plugin = $this->webhookPluginManager->createInstance($definition['id']);

      if (!in_array($topic, $plugin->topics())) {
        continue;
      }

      try {
        $plugin->handle($topic, $shop, $body);
      }
      catch (\Exception $e) {
        Error::logException($this->loggerChannelFactory->get('shopify_app'), $e);
      }
    }
  }

}
